// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AssetManager.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

void AssetManager::init(const std::shared_ptr<Game> &game) {
  assertNotNull(game, "Game is NULL?");
  this->game = game;
}

void AssetManager::update() {
  auto itPending = pendingAssetLoaders.begin();
  while(itPending != pendingAssetLoaders.end()) {
    auto loader = *itPending;
    loader->updateSync();
    loader->updateAsync();
    loader->updateSync();
    if(loader->loaded) {
      finishedAssetLoaders.push_back(loader);
      itPending = pendingAssetLoaders.erase(itPending);
    } else {
      itPending++;
    }
  }
}

void AssetManager::remove(const std::shared_ptr<AssetLoader> loader) {
  auto existing = std::find(
    pendingAssetLoaders.begin(), pendingAssetLoaders.end(), loader
  );
  if(existing != pendingAssetLoaders.end()) {
    pendingAssetLoaders.erase(existing);
  }

  existing = std::find(
    finishedAssetLoaders.begin(), finishedAssetLoaders.end(), loader
  );
  if(existing != finishedAssetLoaders.end()) {
    finishedAssetLoaders.erase(existing);
  }
}

// Disabled because it does not respect scopes
// void AssetManager::removeExisting(const std::string &filename) {
//   auto existing = std::find_if(
//     pendingAssetLoaders.begin(), pendingAssetLoaders.end(),
//     [&](auto &loader) {
//       return loader->name == filename;
//     }
//   );
//   if(existing != pendingAssetLoaders.end()) {
//     pendingAssetLoaders.erase(existing);
//   }

//   existing = std::find_if(
//     finishedAssetLoaders.begin(), finishedAssetLoaders.end(),
//     [&](auto &loader) {
//       return loader->name == filename;
//     }
//   );
//   if(existing != finishedAssetLoaders.end()) {
//     finishedAssetLoaders.erase(existing);
//   }
// }

bool_t AssetManager::isEverythingLoaded() {
  return pendingAssetLoaders.size() == 0;
}

std::shared_ptr<Game> AssetManager::getGame() {
  auto g = game.lock();
  assertNotNull(g, "Game is NULL?");
  return g;
}

AssetManager::~AssetManager() {
  
}