// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "JSONLoader.hpp"
#include "scene/Scene.hpp"

namespace Dawn {
  enum class SceneLoaderState {
    INITIAL,
    LOADING_JSON,
    LOADING_DEPENDENCIES,
    DEPENDENCIES_LOADED,
    PENDING_STAGE,
    DONE
  };

  class SceneLoader :
    public AssetLoader,
    public std::enable_shared_from_this<SceneLoader>
  {
    protected:
      SceneLoaderState state;
      std::shared_ptr<JSONLoader> jsonLoader;
      struct SceneComponentLoadContext ctx;

      /**
       * Loads the dependencies of the scene.
       */
      void setupDependencies();

      /**
       * Scene intializer function to stage the loaded scene.
       * 
       * @param scene Scene that is being staged.
       */
      void sceneInitializer(Scene &scene);

    public:
      const static std::string ASSET_TYPE;

      SceneLoader(
        const std::shared_ptr<AssetManager> assetManager,
        const std::string name
      );
      void updateSync() override;
      void updateAsync() override;
      std::string getAssetType() const override;

      /**
       * Returns the Scene that was loaded, or nullptr if not loaded.
       * 
       * @return The loaded scene.
       */
      std::shared_ptr<Scene> getScene();

      ~SceneLoader();
  };
}