// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"

namespace Dawn {
  class SceneComponentRegistry final {
    public:
      /**
       * Creates a scene component by its type name. Type names are unique and
       * must be registered in order to be constructed by their string name.
       * 
       * @param type Type name of the scene component to create.
       * @param item Scene item that the component belongs to.
       * @return The created scene component.
       */
      static std::shared_ptr<SceneComponent> createComponent(
        const std::string &type,
        const std::shared_ptr<SceneItem> &item
      );
  };
}