// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "CubeMeshComponent.hpp"
#include "display/mesh/CubeMesh.hpp"
#include "component/display/MeshRenderer.hpp"

using namespace Dawn;

void CubeMeshComponent::onInit() {
  if(!mesh) {
    mesh = std::make_shared<Mesh>();
  }

  mesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
  CubeMesh::buffer(
    mesh, glm::vec3(-0.5f, -0.5f, -0.5f), glm::vec3(1.0f, 1.0f, 1.0f), 0, 0
  );

  auto renderer = getItem()->getComponent<MeshRenderer>();
  if(renderer) renderer->mesh = mesh;
}

void CubeMeshComponent::onDispose() {
  mesh = nullptr;
}

void CubeMeshComponent::load(const SceneComponentLoadContext &ctx) {
  if(!mesh) mesh = std::make_shared<Mesh>();

}