
// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scene/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "component/ui/UICanvas.hpp"
#include "asset/loader/TextureLoader.hpp"

#include "component/RPGEntity.hpp"
#include "component/RPGPlayer.hpp"

using namespace Dawn;

void Dawn::rpgScene(Scene &s) {
  auto textureRosa = s.getGame()->assetManager->get<TextureLoader>("rosa.texture");

  while(!s.getGame()->assetManager->isEverythingLoaded()) {
    s.getGame()->assetManager->update();
  }

  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  camera->clipFar = 99999.99f;

  // Player:
  {
    auto ent = s.createSceneItem();
    ent->setLocalPosition(glm::vec3(0, 0, 0.00f));
    auto eEnt = ent->addComponent<RPGEntity>();
    auto ePlyr = ent->addComponent<RPGPlayer>();
    ePlyr->camera = camera;
  }

  // Test Entity
  {
    auto ent = s.createSceneItem();
    ent->setLocalPosition(glm::vec3(-128, -32, -0.01f));
    auto eEnt = ent->addComponent<RPGEntity>();
  }

  auto uiCanvasItem = s.createSceneItem();
  auto uiCanvas = uiCanvasItem->addComponent<UICanvas>();
}